import numpy as np
import matplotlib.pyplot as plt
from peakutils import peak
from struct import unpack
import math
import os

from biosppy.signals import bvp
import pyhrv.tools as tools

from VSRstats.time_domain import time_domain
from VSRstats.frequency_domain import frequency_domain
from VSRstats.nonlinear import nonlinear
from VSRstats.pars_rating import pars_rating
    
class VSR:
    
    def __init__(self, data, hz = 200):
        self.hz = hz

        if type(data) == list:
            self.data = data = np.array(data)
            
        if type(data) == np.ndarray:
            buffer = {}
            self.stats = (self.computeSignal(data) if
                           data.ndim == 1 else self.computeSignals(data))
        else:
            raise TypeError('Signal should be an np.ndarray')

    def __clearkeys(self, obj, keys):
        for k in keys:
            if type(obj) == dict:
                if not (k in obj.keys()):
                    [self.__clearkeys(obj[i], keys) for i in obj.keys()] 
                else:
                    del obj[k]

    def computeSignals(self, signals):
        return np.array([self.computeSignal(s) for s in signals])

    def computeSignal(self, signal):
        obj = {}
            
        obj['time_domain'] = time_domain(signal, self.hz).stats
        
        obj['frequency_domain'] = frequency_domain(signal, self.hz).stats
        self.__clearkeys(obj['frequency_domain'], ['freq', 'power', 'freq_i'])
        
        obj['nonlinear'] = nonlinear(signal, self.hz).stats
        del obj['nonlinear']['poincare']['ellipse']
        del obj['nonlinear']['ACF']
        
        obj['pars_rating'] = pars_rating(signal, self.hz).stats
        
        return obj
        



import numpy as np
import matplotlib.pyplot as plt
from peakutils import peak
from struct import unpack
import biosppy, math, os

from pyhrv.nonlinear import poincare
from statsmodels.tsa import stattools
import matplotlib as mpl
from VSRstats.tools import *

class nonlinear:
    
    def __init__(self, data, hz = 200):
        self.hz = hz
        if type(data) == list:
            data = np.array(data)
            
        if type(data) == np.ndarray:

            self.stats = (self._computeSignal(data) if
                           data.ndim == 1 else self._computeSignals(data))
            
        else:
            raise TypeError('Signal should be an np.ndarray')

    def _poincare(self, nn, peaks):
        res = poincare(nni = nn, rpeaks = peaks, show=False)
        nn_mean = np.mean(nn)
        sd1, sd2 = res['sd1'], res['sd2']
        ellipse = mpl.patches.Ellipse((nn_mean, nn_mean), sd1 * 2, sd2 * 2, angle=-45, fc='k', zorder=1)
        ellipse = ellipse.get_verts()
        ellipse = {'x': ellipse[:,0], 'y': ellipse[:,1]}
        obj = {'sd1': res['sd1'], 'sd2': res['sd2'], 'ellipse': ellipse}
        return obj

    def _ACF(self, signal, lags):
        nlags = lags;
        lags = np.arange(nlags)
        
        acf_x, conflict = stattools.acf(signal, nlags=nlags,
                                        alpha=0.005, fft=False, 
                                        adjusted=False, missing = "none")
        
        return {'lags': lags, 'acf_x': acf_x[lags]}

    def _walk_over(self, obj, key):
        
        keys = []

        if(type(obj[key]) == biosppy.utils.ReturnTuple
           or type(obj[key]) == dict):
            keys = obj[key].keys()
            
        if len(keys) == 0:
            if(type(obj[key]) == biosppy.utils.ReturnTuple):
                return obj[key].as_dict()
            
            elif(type(obj[key]) == np.ndarray):
                return list([i.item() for i in obj[key]])

            else:
                return obj[key]

        res = {}
        for k in keys:
            res[k] = self._walk_over(obj[key], k)
        return res
    
    def _computeSignals(self, signals):
        return np.array([self.computeSignal(s) for s in signals])    

    def _computeSignal(self, signal):
        obj = {}

        # Best min_dist & thres for sphygmogram signal
        peaks = getPeaks(signal, self.hz)
        
        # Ignore un normal signls (with no peaks)
        if(len(peaks) == 0): return obj
            
        nn = nn_intervals(peaks)
        
        # Ignore un normal signls (with no NN)
        if(len(nn) == 0): return

        # Poincare method
        poincare = {'poincare': self._poincare(nn, peaks)}
        obj['poincare'] = self._walk_over(poincare, 'poincare')

        # ACF
        acf = {'ACF': self._ACF(signal, int(len(signal)/2))}
        obj['ACF'] = self._walk_over(acf, 'ACF')

        plt.clf()
        return obj

        



import math, os
import numpy as np
from collections import Counter
from peakutils import peak
from struct import unpack

import pyhrv.time_domain as td
from pyhrv import tools
from scipy import stats

from VSRstats.frequency_domain import frequency_domain
from VSRstats.tools import *

class time_domain:
    
    def __init__(self, data, hz = 200):
        self.hz = hz
        if type(data) == list:
            data = np.array(data)
        if type(data) == np.ndarray:
            self.stats = (self._computeSignal(data) if
                           data.ndim == 1 else self._computeSignals(data))
        else:
            raise TypeError('Signal should be an np.ndarray')

    def _computeSignals(self, signals):
        return np.array([self.computeSignal(s) for s in signals])

    def _cv(self, sdnn, hr_mean):
        return (sdnn / hr_mean) * 100

    def _cc1(self, sdnn, hr_mean):
        return (sdnn / hr_mean) * 100
    
    def _computeSignal(self, signal):
        obj = {}

        # Best min_dist & thres for sphygmogram signal
        peaks = getPeaks(signal, self.hz)
        
        # Ignore un normal signls (with no peaks)
        if(len(peaks) == 0): return obj
            
        nn = nn_intervals(peaks)
        
        # Ignore un normal signls (with no NN)
        if(len(nn) == 0): return

        # Standard
        obj = dict(td.nni_parameters(nn, peaks), **obj)
        obj = dict(td.nni_differences_parameters(nn, peaks), **obj)
        obj = dict(td.sdnn(nn, peaks), **obj)
        obj = dict(td.sdnn_index(nn, peaks), **obj)
        obj = dict(td.sdann(nn, peaks), **obj)
        obj = dict(td.rmssd(nn, peaks), **obj)
        obj = dict(td.sdsd(nn, peaks), **obj)
        obj = dict(td.nn50(nn, peaks), **obj)
        obj = dict(td.nn20(nn, peaks), **obj)
        obj = dict(td.geometrical_parameters(nn, peaks, plot=False), **obj)
        del obj['nni_histogram']
        
        # Additional
        obj = dict({'cv': self._cv(obj['sdnn'], obj['nni_mean'])}, **obj)

        # PEAKS or NN ???
        nn_diff = nni_diff(peaks)
        obj = dict({'MxDMn': max(nn_diff) - min(nn_diff)}, **obj)
        obj = dict({'MxRMn': max(nn_diff) / min(nn_diff)}, **obj)
        obj = dict({'Mo': stats.mode(nn_diff)[0][0]}, **obj)

        counter = Counter(nn_diff)
        idx = list(counter.keys()).index(obj["Mo"])
        obj = dict({'AMo': list(counter.values())[idx]}, **obj)
        obj = dict({'SI': obj['AMo']/(2*obj['Mo']*obj['MxDMn'])}, **obj)

        # Autocorrelation function

        # Frequency stats
        welch = frequency_domain(signal).stats['welch']['params']
        bands = list(welch['fft_bands'].keys())

        obj = dict({'TP': welch['fft_total']}, **obj)
        
        obj = dict({'HF': welch['fft_rel'][bands.index('hf')]}, **obj)
        obj = dict({'LF': welch['fft_rel'][bands.index('lf')]}, **obj)
        obj = dict({'VLF': welch['fft_rel'][bands.index('vlf')]}, **obj)
        obj = dict({'ULF': welch['fft_rel'][bands.index('ulf')]}, **obj)
        
        obj = dict({'HFav': welch['fft_abs'][bands.index('hf')]}, **obj)
        obj = dict({'LFav': welch['fft_abs'][bands.index('lf')]}, **obj)
        obj = dict({'VLFav': welch['fft_abs'][bands.index('vlf')]}, **obj)
        obj = dict({'ULFav': welch['fft_abs'][bands.index('ulf')]}, **obj)
        
        obj = dict({'(LF/HF)av': obj['LFav'] / obj['HFav']}, **obj)
        obj = dict({'IC': obj['LF'] / obj['VLF']}, **obj)

        for k in obj:
            if(math.isnan(obj[k])):
                obj[k] = 0;
        
        return obj



import numpy as np
from peakutils import peak
from struct import unpack
import math
import os

from pyhrv import tools
import matplotlib.pyplot as plt
from VSRstats.tools import *
from VSRstats.time_domain import time_domain

class pars_rating:
    
    def __init__(self, data, hz = 200):
        self.hz = hz
        if type(data) == list:
            data = np.array(data)
        if type(data) == np.ndarray:
            self.stats = (self._computeSignal(data) if
                           data.ndim == 1 else self._computeSignals(data))
        else:
            raise TypeError('Signal should be an np.ndarray')

    def _computeSignals(self, signals):
        return np.array([self.computeSignal(s) for s in signals])

    def _firstParam(self, M):       
        if (M <= 660):
            return 2
        elif (M <= 800):
            return 1
        elif (M >= 1000):
            return -1
        elif (M >= 1200):
            return -2
        return 0

    def _secoundParam(self, sdnn, vr, cv):
        if (sdnn <= 20 and vr <= .1 and cv <= 2):
            return 2
        elif (sdnn >= 100 and vr >= .3 and cv >= 8):
            return 1
        elif (vr >= .1 and vr <= .3):
            return 0
        elif (sdnn >= 100 and vr >= .45 and cv >= 8):
            return -1
        elif (vr >= .6):
            return -2
        return 0

    def _thirdParam(self, vr, am, IN):
        if(vr <= 60 and am >= 80 and IN >= 500):
            return 2
        elif(vr <= 150 and am >= 50 and IN >= 200):
            return 1
        elif(vr >= 300 and am <= 30 and IN <= 50):
            return -1
        elif(vr >= 500 and am <= 15 and IN <= 25):
            return -2
        return 0
    
    def _fourthParam(self, cv):
        if(cv <= 3):
            return 2
        if(cv >= 6):
            return -2
        return 0
    
    def _fifthParam(self, vlf, lf, hf):
        if(vlf >= 70 and lf >= 25 and hf <= 5):
            return 2
        elif(vlf >= 60 and hf <= 20):
            return 1
        elif(vlf <= 40 and hf >= 30):
            return -1
        elif(vlf <= 20 and hf >= 40):
            return -2
        return 0

    def _computeSignal(self, signal):
        obj = {}
        parsCount = 0

        HZstep = 1000 / self.hz
        
        stats = time_domain(signal, self.hz).stats

        peaks = getPeaks(signal, self.hz)
        peaks_diff = nn_intervals(peaks)
        peaks_diff_ms = peaks_diff * HZstep

        M = np.mean(peaks_diff_ms)

        sdnn_ms = np.std(peaks_diff_ms)
        cv = (sdnn_ms / M) * 100

        n, bins, _ = plt.hist(peaks_diff_ms)
        M0 = bins[np.argmax(n)]
        AM0 = (sum(n) / 100) * max(n)
        vr = np.var(bins)
        IN = AM0 / (2 * M0 * vr)

        parsCount += abs(self._firstParam(M))
        parsCount += abs(self._secoundParam(sdnn_ms, vr/M, cv))
        parsCount += abs(self._thirdParam(vr, AM0, IN))       
        parsCount += abs(self._fourthParam(cv))
        parsCount += abs(self._fifthParam(stats['VLF'], stats['LF'], stats['HF']))
        return parsCount



from setuptools import setup, find_packages
from os.path import join, dirname

setup(
    name='VSRstats',
    version='1.26',
    author='Dmitry',
    author_email='semenov.da17@gmail.com',
    packages=['VSRstats'],
    url='http://bsu.ru/',
    description='VSR statstics compute util',
    #long_description=open(join(dirname(__file__), 'README.md')).read(),
    install_requires=['pyhrv', 'numpy', 'peakutils', 'matplotlib', 'argparse'],
)



import os, math
import numpy as np
from struct import unpack
import matplotlib.pyplot as plt
from biosppy.signals import bvp

# Visual show peaks
def showPeaks(signal):
    peaks = getPeaks(signal);
    fig = plt.figure(figsize=(30, 3))
    [ plt.axvline(p, color='r', lw=.5) for p in peaks]
    plt.plot(signal)
    plt.show()

# func to get peaks of signal
def getPeaks(signal, hz = 200):
    return bvp.bvp(signal, hz, show=False)['onsets']

# Read signal from file
def loadSignal(filepath):
    file = open(filepath, "rb").read()
    ufile = unpack('10000H', file)
    return np.array(ufile)

def nn_intervals(peaks):
    return np.array([peaks[i + 1] - peaks[i] for i in range(peaks.size-1)])

def nni_diff(peaks):
    return np.array([abs(peaks[i + 1] - peaks[i]) for i in range(peaks.size-1)])

def filterdSignal(signal, hz = 200):
    return bvp.bvp(signal, hz, show=False)["filtered"]

def normalize(signal):
    return np.array([(x - np.mean(signal))/np.std(signal) for x in signal])
